#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <readline/readline.h>
#include <readline/history.h>

int main(int argc, char *argv[])
{
    char *filename;
    struct stat f_stat;
    int device;
    char *basename, prompt[256];
    char buffer[1024];
    int ret;
    char *command;
    unsigned int l;

    if (argc != 2) {
        fprintf(stderr, "Usage: %s device\n", argv[0]);
        return EXIT_FAILURE;
    }
    filename = argv[1];

    /* Check if it is a socket. */
    ret = lstat(filename, &f_stat);
    if (ret == -1) { perror(filename); return EXIT_FAILURE; }

    /* Open device. */
    if (S_ISSOCK(f_stat.st_mode)) {
        struct sockaddr_un addr;

        device = socket(AF_UNIX, SOCK_STREAM, 0);
        if (device == -1) { perror("socket"); return EXIT_FAILURE; }
        addr.sun_family = AF_UNIX;
        strcpy(addr.sun_path, filename);
        ret = connect(device, (struct sockaddr*) &addr, sizeof addr);
        if (ret == -1) device = -1;
    } else
#if 0
        device = open(filename, O_RDWR|O_NONBLOCK);
#else
        device = open(filename, O_RDWR);
#endif
    if (device == -1) { perror(filename); return EXIT_FAILURE; }

    /* Make prompt. */
    basename = strrchr(filename, '/');
    if (!basename) basename = filename;
    else basename++;
    strncpy(prompt, basename, sizeof prompt - 3);
    prompt[sizeof prompt - 3] = '\0';
    strcat(prompt, "> ");

    /* Shell loop. */
    for (;;) {
        command = readline(prompt);
        if (!command) { puts("quit"); break; }
        if (strcmp(command, "quit") == 0 || strcmp(command, "q") == 0) {
            free(command);
            break;
        }
        add_history(command);
        l = strlen(command);
        command[l] = '\n';           /* not null-terminated */

        /* Question with no command? */
        if (strncmp(command, "?\n", l+1) != 0) {
            ret = write(device, command, l+1);
            if (ret == -1) { perror(filename); return EXIT_FAILURE; }
        }

        /* If we sent a question, we expect an answer. */
        if (l >= 1 && command[l-1] == '?') {

            /* Read the awnser. */
            ret = read(device, buffer, sizeof buffer - 1);
            if (ret == -1) { perror(filename); return EXIT_FAILURE; }
            if (ret == 0) {
                fprintf(stderr, "Got no answer\n");
                continue;
            }

            /* Format with "...\n\0" and output. */
            if (buffer[ret-1] == '\r') buffer[ret-1] = '\n';
            buffer[ret] = '\0';
            fputs(buffer, stdout);
        }
        free(command);
    }

    return EXIT_SUCCESS;
}

/* vim: set sw=4 ts=4: */
